import numpy as np

from pysisyphus.intcoords.Primitive import Primitive


class Bend(Primitive):

    @staticmethod
    def _calculate(coords3d, indices, gradient=False):
        m, o, n = indices
        u_dash = coords3d[m] - coords3d[o]
        v_dash = coords3d[n] - coords3d[o]
        u_norm = np.linalg.norm(u_dash)
        v_norm = np.linalg.norm(v_dash)
        u = u_dash / u_norm
        v = v_dash / v_norm

        cross_vec1 = ( 1, -1, 1)
        cross_vec2 = (-1,  1, 1)

        # Determine second vector for the cross product, to get an
        # orthogonal direction. Eq. (24) in [1]
        uv_parallel = Bend.parallel(u, v)
        if not uv_parallel:
            cross_vec = v
        elif not Bend.parallel(u, cross_vec1):
            cross_vec = cross_vec1
        else:
            cross_vec = cross_vec2

        w_dash = np.cross(u, cross_vec)
        w = w_dash / np.linalg.norm(w_dash)

        angle_rad = np.arccos(u.dot(v))

        if gradient:
            uxw = np.cross(u, w)
            wxv = np.cross(w, v)

            row = np.zeros_like(coords3d)
            #                  |  m  |  n  |  o  |
            # -----------------------------------
            # sign_factor(amo) |  1  |  0  | -1  | first_term
            # sign_factor(ano) |  0  |  1  | -1  | second_term
            first_term = uxw / u_norm
            second_term = wxv / v_norm
            row[m,:] = first_term
            row[o,:] = -first_term - second_term
            row[n,:] = second_term
            row = row.flatten()
            return angle_rad, row
        return angle_rad

    def __str__(self):
        return f"Bend({tuple(self.indices)}"
